package com.usky.dxtop.service.config.rabbiemq;

import com.usky.dxtop.service.config.MqProperties;
import lombok.Data;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

/**
 * @author yq
 * @date 2021/9/8 9:18
 */
@Data
@Configuration
@EnableRabbit
public class CardConfig {


    private static final String NAME = "card_trans_produce";
    @Autowired
    private MqProperties mqProperties;

    private String virtualHost = NAME;
    /**
     * 人员 exchange
     */
    private String cardExchange = NAME;


    /**
     * 人员 queue
     */
    private String cardQueue = NAME;

    /**
     * 人员 route key
     */
    private String cardRouteKey = NAME;


    @Bean
    public DirectExchange cardExchange() {
        DirectExchange exchange = new DirectExchange(cardExchange, true, false);
        exchange.setAdminsThatShouldDeclare(amqpAdmin());
        return exchange;
    }

    @Bean
    Queue cardQueue() {
        Queue queue = new Queue(cardQueue, true);
        queue.setAdminsThatShouldDeclare(amqpAdmin());
        return queue;
    }

    @Bean("cartBin")
    Binding bindingPersonDirect() {
        Binding binding = BindingBuilder.bind(cardQueue()).to(cardExchange()).with(cardRouteKey);
        binding.setAdminsThatShouldDeclare(amqpAdmin());
        return binding;
    }

    @Bean(name = "cardConnectionFactory")
    public ConnectionFactory cardConnectionFactory(){
        return mqProperties.connectionFactory(virtualHost);
    }

    @Bean("cartAdmin")
    AmqpAdmin amqpAdmin() {
        return new RabbitAdmin(cardConnectionFactory());
    }

    @Bean(name = "cardRabbitTemplate")
    public RabbitTemplate rabbitTemplate() {
        RabbitTemplate rabbitTemplate = mqProperties.rabbitTemplate(cardConnectionFactory());
        rabbitTemplate.setMessageConverter(mqProperties.messageConverter());
        return rabbitTemplate;
    }


    @Bean(name = "cardListenerFactory")
    public SimpleRabbitListenerContainerFactory listenerFactory(
            SimpleRabbitListenerContainerFactoryConfigurer configurer) {
        return mqProperties.listenerFactory(configurer,cardConnectionFactory());
    }
}
