package com.usky.dxtop.service.config.rabbiemq;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.usky.dxtop.service.config.MqProperties;
import lombok.Data;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.EnableRabbit;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Primary;

/**
 * @author yq
 * @date 2021/9/8 9:18
 */
@Data
@Configuration
@EnableRabbit
public class PersonConfig {

    private static final String NAME = "profile_trans_produce";

    @Autowired
    private MqProperties mqProperties;

    private String virtualHost = NAME;
    /**
     * 人员 exchange
     */
    private String personExchange = NAME;


    /**
     * 人员 queue
     */
    private String personQueue = NAME;

    /**
     * 人员 route key
     */
    private String personRouteKey = NAME;


    @Bean
    public DirectExchange personExchange() {
        DirectExchange exchange = new DirectExchange(personExchange, true, false);
        exchange.setAdminsThatShouldDeclare(amqpAdmin());
        return exchange;
    }

    @Bean
    Queue personQueue() {
        Queue queue = new Queue(personQueue, true);
        queue.setAdminsThatShouldDeclare(amqpAdmin());
        return queue;
    }

    @Bean("personBin")
    @Primary
    Binding bindingPersonDirect() {
        Binding binding = BindingBuilder.bind(personQueue()).to(personExchange()).with(personRouteKey);
        binding.setAdminsThatShouldDeclare(amqpAdmin());
        return binding;
    }

    @Bean(name = "personConnectionFactory")
    @Primary
    public ConnectionFactory personConnectionFactory(){
        return mqProperties.connectionFactory(virtualHost);
    }

    @Bean("personAdmin")
    @Primary
    AmqpAdmin amqpAdmin() {
        return new RabbitAdmin(personConnectionFactory());
    }

    @Bean(name = "personRabbitTemplate")
    @Primary
    public RabbitTemplate rabbitTemplate() {
        RabbitTemplate rabbitTemplate = mqProperties.rabbitTemplate(personConnectionFactory());
        rabbitTemplate.setMessageConverter(mqProperties.messageConverter());
        return rabbitTemplate;
    }


    @Bean(name = "personListenerFactory")
    @Primary
    public SimpleRabbitListenerContainerFactory listenerFactory(
            SimpleRabbitListenerContainerFactoryConfigurer configurer) {
        return mqProperties.listenerFactory(configurer,personConnectionFactory());
    }

}
